@@ -1,8 +1,55 @@
 # Plugins for the occ function for each data source
+## helper functions
+move_cols <- function(x, y)
+  x[ c(y, names(x)[-sapply(y, function(z) grep(paste0('\\b', z, '\\b'), names(x)))]) ]
+
+emptylist <- function(opts) list(time = NULL, found = NULL, data = data.frame(NULL), opts = opts)
+
+stand_latlon <- function(x){
+  lngs <- c('decimalLongitude', 'decimallongitude', 'Longitude', 'lng', 'longitude', 
+            'decimal_longitude', 'geopoint.lon')
+  lats <- c('decimalLatitude', 'decimallatitude', 'Latitude', 'lat', 'latitude', 
+            'decimal_latitude', 'geopoint.lat')
+  names(x)[ names(x) %in% lngs ] <- 'longitude'
+  names(x)[ names(x) %in% lats ] <- 'latitude'
+  x
+}
+
+add_latlong_if_missing <- function(x) {
+  if (is.null(x$longitude)) x$longitude <- NA
+  if (is.null(x$latitude)) x$latitude <- NA
+  return(x)
+}
+
+stand_dates <- function(dat, from){
+  datevars <- list(gbif = 'eventDate', bison = c('eventDate', 'year'), inat = 'datetime',
+                   ebird = 'obsDt', ecoengine = 'begin_date', vertnet = 'eventdate', 
+                   idigbio = 'datecollected')
+  var <- datevars[[from]]
+  if (from == "bison") {
+    var <- if ( is.null(dat$eventDate) ) "year" else "eventDate"
+  }
+  if ( is.null(dat[[var]]) ) {
+    dat
+  } else {
+    dat[[var]] <- switch(from,
+                         gbif = ymd_hms(dat[[var]], truncated = 3, quiet = TRUE),
+                         bison = ydm_hm(dat[[var]], truncated = 6, quiet = TRUE),
+                         inat = ymd_hms(dat[[var]], truncated = 3, quiet = TRUE),
+                         ebird = ymd_hm(dat[[var]], truncated = 3, quiet = TRUE),
+                         ecoengine = ymd(dat[[var]], truncated = 3, quiet = TRUE),
+                         vertnet = ymd(dat[[var]], truncated = 3, quiet = TRUE),
+                         idigbio = ymd_hms(dat[[var]], truncated = 3, quiet = TRUE)
+    )
+    if (from == "bison") rename(dat, setNames('date', var)) else dat
+  }
+}
+
+## the plugins
 #' @noRd
 foo_gbif <- function(sources, query, limit, geometry, has_coords, callopts, opts) {
   if (any(grepl("gbif", sources))) {
-
+    
     opts$hasCoordinate <- has_coords
     if (!is.null(query)) {
       if (class(query) %in% c("ids", "gbifid")) {
@@ -22,7 +69,7 @@ foo_gbif <- function(sources, query, limit, geometry, has_coords, callopts, opts
     } else {
       query_use <- NULL
     }
-
+    
     if (is.null(query_use) && is.null(geometry)) {
       warning(sprintf("No records found in GBIF for %s", query), call. = FALSE)
       emptylist(opts)
@@ -52,6 +99,7 @@ foo_gbif <- function(sources, query, limit, geometry, has_coords, callopts, opts
             cols <- cols[ cols %in% sort(names(dat)) ]
             dat <- move_cols(x = dat, y = cols)
             dat <- stand_latlon(dat)
+            dat <- add_latlong_if_missing(dat)
             dat <- stand_dates(dat, "gbif")
             list(time = time, found = out$meta$count, data = dat, opts = opts)
           }
@@ -63,41 +111,6 @@ foo_gbif <- function(sources, query, limit, geometry, has_coords, callopts, opts
   }
 }
 
-move_cols <- function(x, y)
-  x[ c(y, names(x)[-sapply(y, function(z) grep(paste0('\\b', z, '\\b'), names(x)))]) ]
-
-emptylist <- function(opts) list(time = NULL, found = NULL, data = data.frame(NULL), opts = opts)
-
-stand_latlon <- function(x){
-  lngs <- c('decimalLongitude', 'decimallongitude', 'Longitude', 'lng', 'longitude', 'decimal_longitude', 'geopoint.lon')
-  lats <- c('decimalLatitude', 'decimallatitude', 'Latitude', 'lat', 'latitude', 'decimal_latitude', 'geopoint.lat')
-  names(x)[ names(x) %in% lngs ] <- 'longitude'
-  names(x)[ names(x) %in% lats ] <- 'latitude'
-  x
-}
-
-stand_dates <- function(dat, from){
-  datevars <- list(gbif = 'eventDate', bison = c('eventDate', 'year'), inat = 'Datetime',
-                   ebird = 'obsDt', ecoengine = 'begin_date', vertnet = 'eventdate')
-  var <- datevars[[from]]
-  if (from == "bison") {
-    var <- if ( is.null(dat$eventDate) ) "year" else "eventDate"
-  }
-  if ( is.null(dat[[var]]) ) {
-    dat
-  } else {
-    dat[[var]] <- switch(from,
-                         gbif = ymd_hms(dat[[var]], truncated = 3, quiet = TRUE),
-                         bison = ydm_hm(dat[[var]], truncated = 6, quiet = TRUE),
-                         inat = ymd_hms(dat[[var]], truncated = 3, quiet = TRUE),
-                         ebird = ymd_hm(dat[[var]], truncated = 3, quiet = TRUE),
-                         ecoengine = ymd(dat[[var]], truncated = 3, quiet = TRUE),
-                         vertnet = ymd(dat[[var]], truncated = 3, quiet = TRUE)
-    )
-    if (from == "bison") rename(dat, setNames('date', var)) else dat
-  }
-}
-
 #' @noRd
 foo_ecoengine <- function(sources, query, limit, geometry, has_coords, callopts, opts) {
   if (any(grepl("ecoengine", sources))) {
@@ -106,10 +119,13 @@ foo_ecoengine <- function(sources, query, limit, geometry, has_coords, callopts,
     opts$georeferenced <- has_coords
     opts$scientific_name <- query
     opts$georeferenced <- TRUE
-    if(!'page_size' %in% names(opts)) opts$page_size <- limit
-    if(!is.null(geometry)){
-      opts$bbox <- if(grepl('POLYGON', paste(as.character(geometry), collapse=" "))){
-        wkt2bbox(geometry) } else { geometry }
+    if (!'page_size' %in% names(opts)) opts$page_size <- limit
+    if (!is.null(geometry)) {
+      opts$bbox <- if (grepl('POLYGON', paste(as.character(geometry), collapse = " "))) {
+        wkt2bbox(geometry) 
+      } else {
+        geometry 
+      }
     }
     # This could hang things if request is super large.  Will deal with this issue
     # when it arises in a usecase
@@ -122,7 +138,7 @@ foo_ecoengine <- function(sources, query, limit, geometry, has_coords, callopts,
     opts$progress <- FALSE
     opts$foptions <- callopts
     out_ee <- tryCatch(do.call(ee_observations, opts), error = function(e) e)
-    if(out_ee$results == 0 || is(out_ee, "simpleError")){
+    if (out_ee$results == 0 || is(out_ee, "simpleError")) {
       warning(sprintf("No records found in Ecoengine for %s", query), call. = FALSE)
       list(time = NULL, found = NULL, data = data.frame(NULL), opts = opts)
     } else{
@@ -132,6 +148,7 @@ foo_ecoengine <- function(sources, query, limit, geometry, has_coords, callopts,
       names(out)[names(out) == 'record'] <- "key"
       out$prov <- rep("ecoengine", nrow(out))
       names(out)[names(out) == 'scientific_name'] <- "name"
+      out <- add_latlong_if_missing(out)
       out <- stand_dates(out, "ecoengine")
       list(time = time, found = out_ee$results, data = out, opts = opts)
     }
@@ -152,7 +169,7 @@ foo_antweb <- function(sources, query, limit, geometry, has_coords, callopts, op
     query <- sub("^ +", "", query)
     query <- sub(" +$", "", query)
 
-    if(length(strsplit(query, " ")[[1]]) == 2) {
+    if (length(strsplit(query, " ")[[1]]) == 2) {
       opts$scientific_name <- query
     } else {
       opts$genus <- query
@@ -170,6 +187,7 @@ foo_antweb <- function(sources, query, limit, geometry, has_coords, callopts, op
       res$prov <- rep("antweb", nrow(res))
       res$name <- query
       res <- stand_latlon(res)
+      res <- add_latlong_if_missing(res)
       list(time = time, found = out$count, data = res, opts = opts)
     }
   } else {
@@ -225,6 +243,7 @@ foo_bison <- function(sources, query, limit, geometry, callopts, opts) {
       dat$prov <- rep("bison", nrow(dat))
       if(bisonfxn == "bison_solr") dat <- rename(dat, c('scientificName' = 'name'))
       dat <- stand_latlon(dat)
+      dat <- add_latlong_if_missing(dat)
       dat <- stand_dates(dat, "bison")
       found <- if(bisonfxn == "bison_solr") out$num_found else out$summary$total
       list(time = time, found = found, data = dat, opts = opts)
@@ -262,6 +281,7 @@ foo_inat <- function(sources, query, limit, geometry, has_coords, callopts, opts
       res$prov <- rep("inat", nrow(res))
       res <- rename(res, c('scientific_name' = 'name'))
       res <- stand_latlon(res)
+      res <- add_latlong_if_missing(res)
       res <- stand_dates(res, "inat")
       list(time = time, found = out$meta$found, data = res, opts = opts)
     }
@@ -295,6 +315,7 @@ foo_ebird <- function(sources, query, limit, callopts, opts) {
       out$prov <- rep("ebird", nrow(out))
       names(out)[names(out) == 'sciName'] <- "name"
       out <- stand_latlon(out)
+      out <- add_latlong_if_missing(out)
       out <- stand_dates(out, "ebird")
       list(time = time, found = NULL, data = out, opts = opts)
     }
@@ -326,7 +347,9 @@ foo_vertnet <- function(sources, query, limit, has_coords, callopts, opts) {
       cols <- cols[ cols %in% sort(names(df)) ]
       df <- move_cols(x = df, y = cols)
       df <- stand_latlon(df)
+      df <- add_latlong_if_missing(df)
       df <- stand_dates(df, "vertnet")
+      names(df) <- tolower(names(df))
       list(time = time, found = as.numeric(gsub(">|<", "", out$meta$matching_records)), data = df, opts = opts)
     }
   } else {
@@ -334,6 +357,65 @@ foo_vertnet <- function(sources, query, limit, has_coords, callopts, opts) {
   }
 }
 
+#' @noRd
+foo_idigbio <- function(sources, query, limit, geometry, has_coords, callopts, opts) {
+  if (any(grepl("idigbio", sources))) {
+    # opts <- limit_alias(opts, "idigbio")
+    time <- now()
+    
+    addopts <- list()
+    if (!is.null(query)) addopts$rq <- list(scientificname = query)
+    if (!is.null(has_coords)) opts$rq$geopoint <- if (has_coords) {
+      list(type = "exists")
+    } else {
+      list(type = "missing")
+    }
+    
+    if (!is.null(geometry)) {
+      if (grepl('POLYGON', paste(as.character(geometry), collapse = " "))) {
+        geometry <- wkt2bbox(geometry)
+      }
+      addopts$rq <- if (is.numeric(geometry) && length(geometry) == 4) {
+        list(geopoint = list(
+          type = "geo_bounding_box", 
+          top_left = list(
+            lat = geometry[4], lon = geometry[1]
+          ), 
+          bottom_right = list(
+            lat = geometry[2], lon = geometry[3]
+          )
+        ))
+      } else {
+        geometry
+      }
+    }
+    
+    if ("rq" %in% names(opts)) {
+      opts$rq <- c(addopts$rq, opts$rq)
+    } else {
+      opts$rq <- addopts$rq
+    }
+    
+    if (is.null(opts$limit)) opts$limit <- limit
+    opts$fields <- "all"
+    
+    out <- tryCatch(do.call(idig_search_records, opts), error = function(e) e)
+    if (is(out, "simpleError")) {
+      warning(sprintf("No records found in iDigBio for %s", query))
+      list(time = NULL, found = NULL, data = data.frame(NULL), opts = opts)
+    } else{
+      out$prov <- rep("idigbio", nrow(out))
+      out <- rename(out, c('scientificname' = 'name'))
+      out <- add_latlong(out, nms = c('geopoint.lon', 'geopoint.lat'))
+      out <- stand_latlon(out)
+      out <- add_latlong_if_missing(out)
+      out <- stand_dates(out, "idigbio")
+      list(time = time, found = attr(out, "itemCount"), data = out, opts = opts)
+    }
+  } else {
+    list(time = NULL, found = NULL, data = data.frame(NULL), opts = opts)
+  }
+}
 
 limit_alias <- function(x, sources, geometry=NULL){
   bisonvar <- if (is.null(geometry)) 'rows' else 'count'
